<?php

namespace Drupal\blizz_bulk_creator\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Class BulkcreateUsageFormStep2.
 *
 * Provides the form portion for the second step of
 * creating new bulkcreate usages.
 *
 * @package Drupal\blizz_bulk_creator\Form
 */
class BulkcreateUsageFormStep2 extends ConfigEntityMultistepFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getStepForm(array $form, FormStateInterface $form_state) {

    return [
      'target_field' => [
        '#type' => 'select',
        '#title' => $this->t('Target field'),
        '#description' => $this->t('Please select the target field referencing the automatically created entities.'),
        '#empty_option' => sprintf('- %s -', $this->t('Please select')),
        '#options' => $this->administrationHelper->getApplicableTargetFields(
          $this->entityTypeManager->getStorage('bulkcreate_configuration')->load($this->store->get('bulkcreate_configuration')),
          $this->store->get('entity_type_id'),
          $this->store->get('bundle')
        ),
        '#required' => TRUE,
      ],
    ];

  }

  /**
   * {@inheritdoc}
   */
  protected function submitStep(array $form, FormStateInterface $form_state) {

    // Get the form input the user filled in.
    $input = $form_state->getUserInput();

    // Save the step data of this step.
    $this->saveData([
      'target_field' => $input['target_field'],
    ]);

    // Set the form to redirect to the next step.
    $form_state->setRedirect('blizz_bulk_creator.bulkcreate_usage.add.step3');

  }

  /**
   * {@inheritdoc}
   */
  protected function isFinalStep() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  protected function getCancelUrl() {
    return new Url('blizz_bulk_creator.bulkcreate_usage.list');
  }

}
